/*
 * This pretends (badly) to be a TypeAlias type node, which would normally be
 * generated by flow-parser from the following code:
 *
 * type aType = {};
 *
 */
import * as esprima from 'esprima';

const program = esprima.parse('var x = \'s\'');
program.body[0].type = 'TypeAlias';

export default program;
